--- old/usr/src/uts/common/fs/sockfs/sockcommon.c	2015-08-17 21:08:03.803597389 +0300
+++ new/usr/src/uts/common/fs/sockfs/sockcommon.c	2015-08-17 21:08:03.809329986 +0300
@@ -475,6 +475,8 @@
 	    offsetof(struct sonode, so_acceptq_node));
 	list_create(&so->so_acceptq_defer, sizeof (struct sonode),
 	    offsetof(struct sonode, so_acceptq_node));
+	list_create(&so->so_pid_list, sizeof (pid_node_t),
+		offsetof(pid_node_t, pn_ref_link));
 	list_link_init(&so->so_acceptq_node);
 	so->so_acceptq_len	= 0;
 	so->so_backlog		= 0;
@@ -490,6 +492,7 @@
 
 	mutex_init(&so->so_lock, NULL, MUTEX_DEFAULT, NULL);
 	mutex_init(&so->so_acceptq_lock, NULL, MUTEX_DEFAULT, NULL);
+	mutex_init(&so->so_pid_list_lock, NULL, MUTEX_DEFAULT, NULL);
 	rw_init(&so->so_fallback_rwlock, NULL, RW_DEFAULT, NULL);
 	cv_init(&so->so_state_cv, NULL, CV_DEFAULT, NULL);
 	cv_init(&so->so_single_cv, NULL, CV_DEFAULT, NULL);
@@ -520,6 +523,7 @@
 
 	list_destroy(&so->so_acceptq_list);
 	list_destroy(&so->so_acceptq_defer);
+	list_destroy(&so->so_pid_list);
 	ASSERT(!list_link_active(&so->so_acceptq_node));
 	ASSERT(so->so_listener == NULL);
 
@@ -535,6 +539,7 @@
 
 	mutex_destroy(&so->so_lock);
 	mutex_destroy(&so->so_acceptq_lock);
+	mutex_destroy(&so->so_pid_list_lock);
 	rw_destroy(&so->so_fallback_rwlock);
 
 	cv_destroy(&so->so_state_cv);
@@ -626,6 +631,7 @@
 sonode_fini(struct sonode *so)
 {
 	vnode_t *vp;
+	pid_node_t *pn;
 
 	ASSERT(so->so_count == 0);
 
@@ -654,6 +660,13 @@
 	if (so->so_filter_top != NULL)
 		sof_sonode_cleanup(so);
 
+	mutex_enter(&so->so_pid_list_lock);
+	while ((pn = list_head(&so->so_pid_list)) != NULL) {
+		list_remove(&so->so_pid_list, pn);
+		kmem_free(pn, sizeof (*pn));
+	}
+	mutex_exit(&so->so_pid_list_lock);
+
 	ASSERT(list_is_empty(&so->so_acceptq_list));
 	ASSERT(list_is_empty(&so->so_acceptq_defer));
 	ASSERT(!list_link_active(&so->so_acceptq_node));
@@ -664,3 +677,48 @@
 	ASSERT(so->so_rcv_head == NULL);
 	ASSERT(so->so_rcv_last_head == NULL);
 }
+
+void
+sonode_insert_pid(struct sonode *so, pid_t pid)
+{
+	pid_node_t	*pn;
+
+	mutex_enter(&so->so_pid_list_lock);
+	for (pn = list_head(&so->so_pid_list);
+	    pn != NULL && pn->pn_pid != pid;
+	    pn = list_next(&so->so_pid_list, pn))
+		;
+
+	if (pn != NULL) {
+		pn->pn_count++;
+	} else {
+		pn = kmem_zalloc(sizeof (*pn), KM_SLEEP);
+		list_link_init(&pn->pn_ref_link);
+		pn->pn_pid = pid;
+		pn->pn_count = 1;
+		list_insert_tail(&so->so_pid_list, pn);
+	}
+	mutex_exit(&so->so_pid_list_lock);
+}
+
+void
+sonode_remove_pid(struct sonode *so, pid_t pid)
+{
+	pid_node_t *pn;
+
+	mutex_enter(&so->so_pid_list_lock);
+	for (pn = list_head(&so->so_pid_list);
+	    pn != NULL && pn->pn_pid != pid;
+	    pn = list_next(&so->so_pid_list, pn))
+		;
+
+	if (pn != NULL) {
+		if (pn->pn_count > 1) {
+				pn->pn_count--;
+		} else {
+			list_remove(&so->so_pid_list, pn);
+			kmem_free(pn, sizeof (*pn));
+		}
+	}
+	mutex_exit(&so->so_pid_list_lock);
+}
